"""
Classes related to Atlas clusters.

Supports the creation and configuration of Atlas clusters of various types.

Enums are used in order to minimize invalid configuration values.

"""
from enum import Enum
from typing import List, Optional
from datetime import datetime
import pytz
import uuid

from atlasapi.lib import ProviderName, MongoDBMajorVersion, ClusterType

FORMAT = '%Y-%m-%dT%H:%M:%SZ'


# Enums
class VolumeTypes(Enum):
    """
    The volume types available on atlas
    """
    STANDARD = "Standard"
    PROVISIONED = 'Provisioned'


class ClusterStates(Enum):
    """
    The states of Atlas clusters. RO attribute.

    """
    IDLE = 'Idle'
    CREATING = 'Creating'
    UPDATING = 'Updating'
    DELETING = 'Deleting'
    DELETED = 'Deleted'
    REPAIRING = 'Repairing'
    UNKNOWN = 'Unknown'


class InstanceSizeName(Enum):
    """
    The Atlas instance sizes, which are equivalent across all providers.

    NVME suggixed types provide loval NVME disk.


    """
    M0 = 'M0'
    M2 = 'M2'
    M5 = 'M5'
    M10 = 'M10'
    M20 = 'M20'
    M30 = 'M30'
    M40 = 'M40'
    R40 = 'R40'
    M40_NVME = 'M40 NVME'
    M50 = 'M50'
    R50 = 'R50'
    M50_NVME = 'M50 NVME'
    M60 = 'M60'
    R60 = 'R60'
    M60_NVME = 'M60 NVME'
    M80 = 'M80'
    R80 = 'R80'
    M80_NVME = 'M80 NVME'
    M100 = 'M100'
    M140 = 'M140'
    M200 = 'M200'
    M200_NVME = 'M200_NVME'
    R200 = 'R200'
    M300 = 'M300'
    R300 = 'R300'
    M400 = 'M400'
    R400 = 'R400'
    M400_NVME = 'M400_NVME'
    R700 = 'R700'


class TLSProtocols(Enum):
    TLS1_0 = 'TLS1_0'
    TLS1_1 = 'TLS1_1'
    TLS1_2 = 'TLS1_2'
    TLS1_3 = 'TLS1_3'  # 4.0 And Above Only


# Classes

class RegionConfig(object):
    """Configuration object for each region.

                       Allows for the configuration of each region independently. Includes sane defaults.

                       Args:
                           electable_node_count (int):  Number of electable nodes.
                           priority: Priority of the region in the replica set.
                           read_only_node_count: Count of read_only nodes.
                           analytics_node_count:  Count of analytics nodes.
    """
    def __init__(self,
                 electable_node_count: int = 3,
                 priority: int = 7,
                 read_only_node_count: int = 0,
                 analytics_node_count: int = 0):
        """

        Args:
            read_only_node_count (int):
        """
        self.analyticsNodes = analytics_node_count
        self.electableNodes = electable_node_count
        self.priority = priority
        self.readOnlyNodes = read_only_node_count


class ReplicationSpecs(object):
    def __init__(self, id: Optional[str] = uuid.uuid4().__str__(),
                 num_shards: Optional[int] = 1,
                 zone_name: Optional[str] = None,
                 regions_config: Optional[dict] = None):
        """
        Configuration of each region in the cluster. Each element in this document represents a region where Atlas
        deploys your cluster.

        NOTE: A ReplicationSpecs object is found in Atlas replicationSpecs

        :param id:
        :param num_shards:
        :param zone_name:
        :param regions_config:
        """
        self.regions_config = regions_config
        self.zone_name = zone_name
        self.num_shards = num_shards
        self.id = id

    @classmethod
    def from_dict(cls, data_dict: dict):
        id = data_dict.get('id', None)
        regions_config = data_dict.get('regionsConfig', None)
        zone_name = data_dict.get('zoneName', None)
        num_shards = data_dict.get('numShards', None)
        return cls(id, num_shards, zone_name, regions_config)

    def as_dict(self):
        return_dict = self.__dict__
        return_dict['numShards'] = self.num_shards
        return_dict.__delitem__('num_shards')
        return_dict['regionsConfig'] = self.regions_config
        return_dict.__delitem__('regions_config')
        if self.zone_name is not None:
            return_dict['zoneName'] = self.zone_name
        return_dict.__delitem__('zone_name')

        return return_dict

    def as_create_dict(self):
        out_dict = self.as_dict()
        out_dict.__delitem__('id')

        return out_dict


class ProviderSettings(object):
    def __init__(self,
                 size: InstanceSizeName = InstanceSizeName.M10,
                 provider: ProviderName = ProviderName.AWS,
                 region: str = 'US_WEST_1',
                 autoScaling: Optional[dict] = None,
                 diskIOPS: int = None,
                 encryptEBSVolume: bool = True,
                 volumeType: VolumeTypes = VolumeTypes.STANDARD
                 ):
        """
        Configuration for the provisioned servers on which MongoDB runs.


        :param size: Name of the cluster tier used for the Atlas cluster.
        :param provider: Cloud service provider on which the servers are provisioned.
        :param region: Physical location of your MongoDB cluster. The region you choose can affect network latency for
                        clients accessing your databases.
        :param autoScaling: Contains the compute field which specifies the range of instance sizes to which your cluster
                can scale.
        :param diskIOPS: Maximum input/output operations per second (IOPS) the system can perform.
        :param encryptEBSVolume: AWS only. If enabled, the Amazon EBS encryption feature encrypts the server’s root
                volume for both data at rest within the volume and for data moving between the volume and the cluster.
        :param volumeType: The type of AWS volume.
        """
        self.volumeType = volumeType
        self.encryptEBSVolume = encryptEBSVolume
        self.diskIOPS = diskIOPS
        self.autoScaling = autoScaling
        self.instance_size_name: InstanceSizeName = size
        self.provider_name: ProviderName = provider
        self.region_name: str = region

    @classmethod
    def from_dict(cls, data_dict: dict):
        try:
            size = InstanceSizeName[data_dict.get('instanceSizeName', None)]
        except ValueError:
            size = None
        try:
            provider = ProviderName[data_dict.get('providerName', None)]
        except ValueError:
            provider = None
        region = data_dict.get('regionName', None)
        autoscaling = data_dict.get('autoScaling', None)
        diskIOPS = data_dict.get('diskIOPS', None)
        encryptEBSVolume = data_dict.get('encryptEBSVolume', None)
        volumeType = data_dict.get('volumeType', None)
        return cls(size, provider, region, autoscaling, diskIOPS, encryptEBSVolume, volumeType)

    def as_dict(self) -> dict:
        out_dict = self.__dict__
        if type(self.instance_size_name) != str:
            out_dict['instanceSizeName'] = self.instance_size_name.name
        if type(self.provider_name) != str:
            out_dict['providerName'] = self.provider_name.name
        try:
            out_dict['volumeType'] = self.volumeType.name
        except AttributeError:
            out_dict['volumeType'] = self.volumeType

        out_dict['regionName'] = self.region_name
        del out_dict['provider_name']
        del out_dict['instance_size_name']
        del out_dict['region_name']

        return out_dict


class ClusterConfig(object):
    """Stores the Atlas Cluster Config, is sent back to the API for any reconfiguations.

        https://docs.atlas.mongodb.com/reference/api/clusters-get-one/#http-response-elements

         Args:
            backup_enabled:
            cluster_type:
            disk_size_gb:
            name:
            mongodb_major_version:
            mongodb_version:
            num_shards:
            mongo_uri:
            mongo_uri_updated:
            mongo_uri_with_options:
            paused:
            pit_enabled:
            replication_factor:
            state_name:
            autoscaling:
            replication_specs:
            srv_address:
            providerSettings:
            links:

    """
    def __init__(self, backup_enabled: bool = False,
                 cluster_type: ClusterType = ClusterType.REPLICASET,
                 disk_size_gb: int = 32,
                 name: str = None,
                 mongodb_major_version: MongoDBMajorVersion = MongoDBMajorVersion.v4_4,
                 mongodb_version: Optional[str] = None,
                 num_shards: int = 1,
                 mongo_uri: Optional[str] = None,
                 mongo_uri_updated: Optional[str] = None,
                 mongo_uri_with_options: Optional[str] = None,
                 paused: bool = False,
                 pit_enabled: bool = False,
                 replication_factor: Optional[int] = None,
                 state_name: Optional[ClusterStates] = None,
                 autoscaling: dict = None,
                 replication_specs: ReplicationSpecs = None,
                 srv_address: Optional[str] = None,
                 providerSettings: Optional[ProviderSettings] = None,
                 links: list = None,
                 id: Optional[str] = None
                 ) -> None:
        self.id: Optional[str] = id
        self.providerSettings: Optional[ProviderSettings] = providerSettings
        self.backup_enabled: bool = backup_enabled
        self.cluster_type: ClusterType = cluster_type
        self.disk_size_gb: int = disk_size_gb
        self.name: Optional[str] = name
        self.mongodb_major_version: MongoDBMajorVersion = mongodb_major_version
        self.mongodb_version: Optional[str] = mongodb_version
        self.num_shards: int = num_shards
        self.mongo_uri: Optional[str] = mongo_uri
        self.mongo_uri_updated: Optional[datetime] = mongo_uri_updated
        self.mongod_uri_with_options: Optional[str] = mongo_uri_with_options
        self.paused: bool = paused
        self.pit_enabled: bool = pit_enabled
        self.replication_factor: Optional[int] = replication_factor
        self.state_name: Optional[ClusterStates] = state_name
        self.autoscaling: dict = autoscaling
        if type(replication_specs) == list:
            self.replication_specs: List[ReplicationSpecs] = replication_specs
        else:
            self.replication_specs: List[ReplicationSpecs] = [replication_specs]
        self.links: Optional[list] = links
        self.srv_address: str = srv_address

    @classmethod
    def fill_from_dict(cls, data_dict: dict):
        replication_specs = data_dict.get('replicationSpecs', {})
        backup_enabled = data_dict.get('backupEnabled', None)
        disk_size_gb = data_dict.get('diskSizeGB', None)
        name = data_dict.get('name', None)
        num_shards = data_dict.get('numShards', 1)
        clusterType = data_dict.get('clusterType', None)
        cluster_type = ClusterType[clusterType]
        mongo_uri = data_dict.get('mongoURI', None)
        mongo_uri_with_options = data_dict.get('mongoURIWithOptions', None)
        try:
            mongo_uri_updated = datetime.strptime(data_dict.get('mongoURIUpdated', None), FORMAT).astimezone(
                tz=pytz.UTC)
        except ValueError:
            mongo_uri_updated = None
        try:
            mongodb_major_version = MongoDBMajorVersion(data_dict.get('mongoDBMajorVersion'))
        except ValueError:
            mongodb_major_version = MongoDBMajorVersion.vX_x
        mongodb_version = data_dict.get('mongoDBVersion', None)
        paused = data_dict.get('paused', False)
        pit_enabled = data_dict.get('pitEnabled', False)
        replication_factor = data_dict.get('replicationFactor', None)
        state_name = ClusterStates[data_dict.get('stateName', ClusterStates.UNKNOWN)]
        autoscaling = data_dict.get('autoScaling', {})

        if len(replication_specs) > 1:
            raise ValueError('We are limited to a single replication spec.')
        replication_specs = replication_specs[0]
        repl_spec_obj = ReplicationSpecs.from_dict(replication_specs)
        replication_specs = repl_spec_obj
        provider_settings_dict = data_dict.get('providerSettings', None)
        srv_address = data_dict.get('srvAddress', None)
        providerSettings = ProviderSettings.from_dict(provider_settings_dict)
        links = data_dict.get('links', [])
        id = data_dict.get('id', None)
        return cls(backup_enabled, cluster_type, disk_size_gb, name, mongodb_major_version, mongodb_version,
                   num_shards, mongo_uri, mongo_uri_updated, mongo_uri_with_options, paused, pit_enabled,
                   replication_factor, state_name, autoscaling, replication_specs,
                   srv_address, providerSettings, links,id)

    def as_dict(self) -> dict:
        return_dict = self.__dict__
        try:
            return_dict['clusterType'] = self.cluster_type.name
            return_dict.__delitem__('cluster_type')
            return_dict['mongoDBMajorVersion'] = self.mongodb_major_version.value
            return_dict.__delitem__('mongodb_major_version')
            return_dict['replicationSpecs'] = [self.replication_specs[0].as_dict()]
            return_dict.__delitem__('replication_specs')

        except (KeyError, AttributeError):
            pass

        try:
            return_dict['stateName'] = self.state_name.name
        except AttributeError:
            pass
        try:
            return_dict.__delitem__('state_name')
        except KeyError:
            pass
        try:
            if type(return_dict['providerSettings']) != dict:
                return_dict['providerSettings'] = self.providerSettings.as_dict()
            else:
                return_dict['providerSettings'] = self.providerSettings
        except AttributeError:
            return_dict['providerSettings'] = self.providerSettings
        try:
            return_dict.__delitem__('replication_factor')  # THis has been deprecated, so removing from dict output
            return_dict['mongoDBVersion'] = self.mongodb_version
            return_dict.__delitem__('mongodb_version')
            return_dict['srvAddress'] = self.srv_address
            return_dict.__delitem__('srv_address')
            return_dict['mongoURI'] = self.mongo_uri
            return_dict.__delitem__('mongo_uri')
            return_dict['mongoURIWithOptions'] = self.mongod_uri_with_options
            return_dict.__delitem__('mongod_uri_with_options')
            return_dict['diskSizeGB'] = self.disk_size_gb
            return_dict.__delitem__('disk_size_gb')
            return_dict['pitEnabled'] = self.pit_enabled
            return_dict.__delitem__('pit_enabled')
            return_dict['numShards'] = self.num_shards
            return_dict.__delitem__('num_shards')
            return_dict['mongoURIUpdated'] = self.mongo_uri_updated
            return_dict.__delitem__('mongo_uri_updated')
            return_dict['backupEnabled'] = self.backup_enabled
            return_dict.__delitem__('backup_enabled')
        except KeyError:
            pass

        return return_dict

    def as_create_dict(self) -> dict:
        """
        Returns the config object in a format acceptable for the POST (create) endpoint.

        Removes properties which are read-only.

        TODO: Refactor to identify which properties are RO in the spec, and automatically loop through and remove.

        :return: dict: A dict containing a valid create object for the POST endpoint.
        """
        out_dict = self.as_dict()
        try:
            out_dict.pop('numShards', None)
            out_dict.pop('mongoURI', None)
            out_dict.pop('mongoDBVersion', None)
            out_dict.pop('mongoURIUpdated', None)
            out_dict.pop('mongoURIWithOptions', None)
            out_dict.pop('paused', None)
            out_dict.pop('srvAddress', None)
            out_dict.pop('links', None)
            out_dict.pop('state_name', None)
        except KeyError:
            pass
        try:
            out_dict['replicationSpecs'][0].__delitem__('id')
        except KeyError:
            pass
        return out_dict

    def as_modify_dict(self) -> dict:
        """
        Returns the config object in a format acceptable for the PATCH (modify) endpoint.

        Removes properties which are read-only.

        TODO: Refactor to identify which properties are RO in the spec, and automatically loop through and remove.

        :return: dict: A dict containing a valid create object for the POST endpoint.
        """
        out_dict = self.as_dict()
        out_dict.pop('stateName', None)
        out_dict.pop('numShards', None)
        out_dict.pop('mongoURI', None)
        out_dict.pop('mongoDBVersion', None)
        out_dict.pop('mongoURIUpdated', None)
        out_dict.pop('mongoURIWithOptions', None)
        out_dict.pop('paused', None)
        out_dict.pop('srvAddress', None)
        out_dict.pop('links', None)
        out_dict.pop('state_name', None)

        return out_dict


class ShardedClusterConfig(ClusterConfig):
    def __init__(self, backup_enabled: bool = False,
                 cluster_type: ClusterType = ClusterType.REPLICASET,
                 disk_size_gb: int = 32,
                 name: str = None,
                 mongodb_major_version: MongoDBMajorVersion = MongoDBMajorVersion.v4_0,
                 mongodb_version: Optional[str] = None,
                 num_shards: int = 1,
                 mongo_uri: Optional[str] = None,
                 mongo_uri_updated: Optional[str] = None,
                 mongo_uri_with_options: Optional[str] = None,
                 paused: bool = False,
                 pit_enabled: bool = False,
                 replication_factor: Optional[int] = None,
                 state_name: Optional[ClusterStates] = None,
                 autoscaling: dict = {},
                 replication_specs: list = [],
                 srv_address: Optional[str] = None,
                 providerSettings: Optional[ProviderSettings] = None,
                 links: list = None,
                 id: Optional[str] = None):
        super().__init__(backup_enabled, cluster_type, disk_size_gb, name, mongodb_major_version, mongodb_version,
                         num_shards, mongo_uri, mongo_uri_updated, mongo_uri_with_options, paused, pit_enabled,
                         replication_factor, state_name, autoscaling, [replication_specs], srv_address,
                         providerSettings, links, id)

    def as_dict(self) -> dict:
        return_dict = self.__dict__
        return_dict['clusterType'] = self.cluster_type.name
        return_dict.__delitem__('cluster_type')
        return_dict['mongoDBMajorVersion'] = self.mongodb_major_version.value
        return_dict.__delitem__('mongodb_major_version')
        try:
            return_dict['replicationSpecs'] = [self.replication_specs[0].as_dict()]
        except AttributeError:
            return_dict['replicationSpecs'] = [self.replication_specs[0][0].as_dict]
        return_dict.__delitem__('replication_specs')
        try:
            return_dict['stateName'] = self.state_name.name
            return_dict.__delitem__('state_name')
        except AttributeError:
            pass
        return_dict['providerSettings'] = self.providerSettings.as_dict()
        return_dict.__delitem__('replication_factor')  # THis has been deprecated, so removing from dict output
        return_dict['mongoDBVersion'] = self.mongodb_version
        return_dict.__delitem__('mongodb_version')
        return_dict['srvAddress'] = self.srv_address
        return_dict.__delitem__('srv_address')
        return_dict['mongoURI'] = self.mongo_uri
        return_dict.__delitem__('mongo_uri')
        return_dict['mongoURIWithOptions'] = self.mongod_uri_with_options
        return_dict.__delitem__('mongod_uri_with_options')
        return_dict['diskSizeGB'] = self.disk_size_gb
        return_dict.__delitem__('disk_size_gb')
        return_dict['pitEnabled'] = self.pit_enabled
        return_dict.__delitem__('pit_enabled')
        return_dict['numShards'] = self.num_shards
        return_dict.__delitem__('num_shards')
        return_dict['mongoURIUpdated'] = self.mongo_uri_updated
        return_dict.__delitem__('mongo_uri_updated')
        return_dict['backupEnabled'] = self.backup_enabled
        return_dict.__delitem__('backup_enabled')

        return return_dict


class AtlasBasicReplicaSet(object):
    """Helper object for the creation of a basic replica set with default options.

    Only the cluster name is required.

    Other parameters will default to An M10 cluster on AWS US_WEST_2 running 4.0 with a 10 GB disk.

            Args:
                name: The name given to the cluster/replica set.
                size: The InstanceSizeName of the cluster/replica set
                disk_size: Size of disks on all members
                provider: The Cloud provider
                region: The region in the cloud provider
                version: The MongoDB major version
            """
    def __init__(self, name: str,
                 size: InstanceSizeName = InstanceSizeName.M10,
                 disk_size: int = 10,
                 provider: ProviderName = ProviderName.AWS,
                 region: str = 'US_WEST_2',
                 version: MongoDBMajorVersion = MongoDBMajorVersion.v4_0,
                 ) -> None:

        provider_settings = ProviderSettings(size=size,
                                             provider=provider, region=region)
        regions_config = RegionConfig()
        replication_specs = ReplicationSpecs(regions_config={region: regions_config.__dict__})

        self.config: ClusterConfig = ClusterConfig(disk_size_gb=disk_size,
                                                   name=name,
                                                   mongodb_major_version=version,
                                                   providerSettings=provider_settings,
                                                   replication_specs=replication_specs)
        self.config_running = None
        self.config_pending = None

    def as_dict(self):
        cluster_config_dict = self.config.providerSettings.as_dict()
        raw_dict = self.__dict__
        raw_dict['providerSettings'] = cluster_config_dict
        return raw_dict


class AdvancedOptions(object):
    """Container for Atlas Cluster Advanced options

            Args:
                failIndexKeyTooLong: When true, documents can only be updated or inserted if, for all indexed fields on
                    the target collection, the corresponding index entries do not exceed 1024 bytes.When false, mongod writes
                    documents that breach the limit but does not index them.
                javascriptEnabled:  When true, the cluster allows execution of operations that perform server-side
                    executions of JavaScript. When false, the cluster disables execution of those operations.
                minimumEnabledTlsProtocol: The minimum Transport Layer Security (TLS) version the cluster accepts for
                    incoming connections.
                noTableScan: When true, the cluster disables the execution of any query that requires a collection scan
                    to return results. When false, the cluster allows the execution of those operations.
                oplogSizeMB: The custom oplog size of the cluster. A value of null indicates that the cluster uses the
                    default oplog size calculated by Atlas.
                sampleSizeBIConnector: Number of documents per database to sample when gathering schema information.
                sampleRefreshIntervalBIConnector: Interval in seconds at which the mongosqld process re-samples data to
                    create its relational schema.
            """
    def __init__(self,
                 failIndexKeyTooLong: Optional[bool]  = None,
                 javascriptEnabled: Optional[bool] = None,
                 minimumEnabledTlsProtocol: Optional[TLSProtocols] = None,
                 noTableScan: Optional[bool] = None,
                 oplogSizeMB: Optional[int] = None,
                 sampleSizeBIConnector: Optional[int] = None,
                 sampleRefreshIntervalBIConnector: Optional[int] = None):

        self.sampleRefreshIntervalBIConnector = sampleRefreshIntervalBIConnector
        self.sampleSizeBIConnector = sampleSizeBIConnector
        self.oplogSizeMB = oplogSizeMB
        self.noTableScan = noTableScan
        self.minimumEnabledTlsProtocol = minimumEnabledTlsProtocol
        self.javascriptEnabled = javascriptEnabled
        self.failIndexKeyTooLong = failIndexKeyTooLong

    @classmethod
    def fill_from_dict(cls, data_dict: dict):
        """
        Fills the advanced options object from an Atlas Dict

        :param data_dict: A dict as returned from Atlas
        :return:
        """
        failIndexKeyTooLong = data_dict.get('failIndexKeyTooLong', None)
        javascriptEnabled = data_dict.get('javascriptEnabled', None)
        minimumEnabledTlsProtocol = data_dict.get('minimumEnabledTlsProtocol', None)
        noTableScan = data_dict.get('noTableScan', None)
        oplogSizeMB = data_dict.get('oplogSizeMB', None)
        sampleSizeBIConnector = data_dict.get('sampleSizeBIConnector', None)
        sampleRefreshIntervalBIConnector = data_dict.get('sampleRefreshIntervalBIConnector', None)
        if data_dict.get('minimumEnabledTlsProtocol', None):
            minimumEnabledTlsProtocol = TLSProtocols[data_dict.get('minimumEnabledTlsProtocol', None)]

        return cls(failIndexKeyTooLong, javascriptEnabled, minimumEnabledTlsProtocol, noTableScan, oplogSizeMB,
                   sampleSizeBIConnector, sampleRefreshIntervalBIConnector)

    @property
    def as_dict(self) -> dict:
        """
        Returns a json-able dict of only non-null properties.

        :return:
        """
        return_dict = dict()
        for key, value in self.__dict__.items():
            if value is not None:
                if isinstance(value, Enum):
                    return_dict[key] = value.name
                else:
                    return_dict[key] = value
            else:
                pass
        return return_dict






from enum import Enum
from isodate import isodatetime
from datetime import datetime
from typing import Optional


class Project:
    def __init__(self, name: str, org_id: str, created_date: Optional[datetime] = None,
                 cluster_count: Optional[int] = None, id: Optional[str] = None, links: list = None,
                 with_default_alert_settings: Optional[bool] = True,
                 project_owner_id: str = None) -> None:
        """A single Atlas Project/Group

        Groups and projects are synonymous terms. Your {GROUP-ID} is the same as your project ID. For existing groups,
         your group/project ID remains the same. The resource and corresponding endpoints use the term groups.

        Args:
            id (str): The unique identifier of the project. You can use this value for populating the {GROUP-ID}
            parameter of other Atlas Administration API endpoints.
            name (str): The name of the project. You can use this value for populating the {GROUP-NAME} parameter of the /groups/byName/{GROUP-NAME} endpoint.
            links (list): One or more uniform resource locators that link to sub-resources and/or related resources. The Web Linking Specification explains the relation-types between URLs.
            org_id (str): The unique identifier of the Atlas organization to which the project belongs.
            created_date (Optional[datetime]): The ISO-8601-formatted timestamp of when Atlas created the project.
            cluster_count (int): The number of Atlas clusters deployed in the project.
            with_default_alert_settings (bool): Flag that indicates whether to create the new project with the default alert settings enabled. This parameter defaults to true.

        """
        self.project_owner_id: Optional[str] = project_owner_id
        self.with_default_alert_settings: bool = with_default_alert_settings
        self.cluster_count: Optional[int] = cluster_count
        self.created_date: Optional[datetime] = created_date
        self.org_id: str = org_id
        self.links: Optional[list] = links
        self.name: str = name
        self.id: Optional[str] = id

    @classmethod
    def for_create(cls, name: str, org_id: str, with_default_alert_settings: bool = True, project_owner_id: str = None):
        """
        Creates a new Project object for use in creating a new project.

        Only name and org_id are required.

        Args:
            project_owner_id (str): Unique 24-hexadecimal digit string that identifies the Atlas user account to be granted the Project Owner role on the specified project. If you set this parameter, it overrides the default value of the oldest Organization Owner.
            name (str): The name of the project. You can use this value for populating the {GROUP-NAME} parameter of the /groups/byName/{GROUP-NAME} endpoint.
            org_id (str): The unique identifier of the Atlas organization to which the project belongs.
            with_default_alert_settings (bool): Flag that indicates whether to create the new project with the default alert settings enabled. This parameter defaults to true.

        Returns: None

        """
        return cls(name=name, org_id=org_id, with_default_alert_settings=with_default_alert_settings,
                   project_owner_id=project_owner_id)

    @classmethod
    def from_dict(cls, data_dict):
        """
        Creates a Project object from a passed dict, in the format of the Atlas API.

        Args:
            data_dict (dict): A dictionary in the format of the Atlas API.

        Returns: None

        """
        return cls(id=data_dict.get("id"), name=data_dict.get("name"),
                   links=data_dict.get("links", []), org_id=data_dict.get("orgId"),
                   created_date=data_dict.get("created"), cluster_count=data_dict.get("clusterCount"))

    @property
    def create_dict(self) -> dict:
        """
        A dictionary in the format Atlas API "create project expects"

        Returns: A dictionary in the format Atlas API "create project expects"

        """
        return dict(name=self.name, orgId = self.org_id, withDefaultAlertsSettings=self.with_default_alert_settings)


class ProjectSettings:
    def __init__(self, is_collect_db_stats: Optional[bool] = None, is_data_explorer: Optional[bool] = None,
                 is_performance_advisor: Optional[bool] = None, is_realtime_perf: Optional[bool] = None,
                 is_schema_advisor: Optional[bool] = None):
        """Holds Project/Group settings.

        Args:
            is_collect_db_stats (Optional[bool]): Flag that indicates whether statistics in cluster metrics collection is enabled for the project.
            is_data_explorer (Optional[bool]): Flag that indicates whether Data Explorer is enabled for the project. If enabled, you can query your database with an easy to use interface.
            is_performance_advisor (Optional[bool]): Flag that indicates whether Performance Advisor and Profiler is enabled for the project. If enabled, you can analyze database logs to recommend performance improvements.
            is_realtime_perf (Optional[bool]): Flag that indicates whether Real Time Performance Panel is enabled for the project. If enabled, you can see real time metrics from your MongoDB database.
            is_schema_advisor (Optional[bool]): Flag that indicates whether Schema Advisor is enabled for the project. If enabled, you receive customized recommendations to optimize your data model and enhance performance.
        """
        self.is_schema_advisor: Optional[bool] = is_schema_advisor
        self.is_realtime_perf: Optional[bool] = is_realtime_perf
        self.is_performance_advisor: Optional[bool] = is_performance_advisor
        self.is_data_explorer: Optional[bool] = is_data_explorer
        self.is_collect_db_stats: Optional[bool] = is_collect_db_stats

    @classmethod
    def from_dict(cls,data_dict: dict):
        return cls(
            bool(data_dict.get("isCollectDatabaseSpecificsStatisticsEnabled", False)),
            bool(data_dict.get("isDataExplorerEnabled", False)),
            bool(data_dict.get("isPerformanceAdvisorEnabled", False)),
            bool(data_dict.get("isRealtimePerformancePanelEnabled", False)),
            bool(data_dict.get("isSchemaAdvisorEnabled", False)),
        )


"""
test_dict = {
    "clusterCount": 2,
    "created": "2016-07-14T14:19:33Z",
    "id": "5a0a1e7e0f2912c554080ae6",
    "links": [],
    "name": "DocsFeedbackGroup",
    "orgId": "5a0a1e7e0f2912c554080adc"
}

print(Project.for_create(org_id='34', name='test_project').as_create_dict)
"""


# Copyright (c) 2019 Matthew G. Monteleone
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ipaddress import IPv4Address, IPv4Network
from pprint import pprint
from typing import Optional
import logging
from datetime import datetime
from isodate import parse_datetime
class WhitelistEntry(object):
    def __init__(self, cidrBlock: str = None, comment: str = None, ipAddress: str = None, links: list = None,
                 last_used: str = None, count: int = None, last_used_address: str = None):
        """
        For a single whitelist entry. Contains a bit of helper intelligence for ip addresses.

        :param cidrBlock:
        :param comment:
        :param ipAddress:
        :param links:
        """
        self.last_used_address: Optional[IPv4Address]  = None
        try:
            self.last_used_address = IPv4Address(last_used_address)
        except Exception:
            logging.warning('No last used address')

        self.count: Optional[int] = count
        self.last_used: Optional[datetime] = None
        try:
            self.last_used = parse_datetime(last_used)
        except Exception:
            logging.warning('Could not get last used date.')
        self.links = links
        self.ipAddress = ipAddress
        self.comment = comment
        self.cidrBlock = cidrBlock
        try:
            self.cidrBlockObj: IPv4Network = IPv4Network(self.cidrBlock)
        except Exception:
            self.cidrBlockObj = None
        try:
            self.ipAddressObj: IPv4Address = IPv4Address(self.ipAddress)
        except Exception:
            self.ipAddressObj = None

    @classmethod
    def fill_from_dict(cls, data_dict: dict):
        """
        Fills the object from the standard Atlas API dictionary.
        :param data_dict:
        :return:
        """
        cidrBlock = data_dict.get('cidrBlock', None)
        comment = data_dict.get('comment', None)
        ipAddress = data_dict.get('ipAddress', None)
        links = data_dict.get('links', None)
        last_used = data_dict.get('lastUsed', None)
        count = data_dict.get('count', 0)
        last_used_address = data_dict.get('lastUsedAddress', None)

        return cls(cidrBlock=cidrBlock, comment=comment, ipAddress=ipAddress, links=links,
                   last_used=last_used,count=count,last_used_address=last_used_address)

    def as_dict(self) -> dict:
        """
        Dumps obj as a json valid dict.
        :return:
        """
        orig_dict = self.__dict__
        orig_dict.__delitem__('ipAddressObj')
        orig_dict.__delitem__('cidrBlockObj')
        try:
            orig_dict['last_used_address'] = self.last_used_address.__str__()
        except Exception:
            pass
        return orig_dict



# Copyright (c) 2022 Matthew G. Monteleone
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Specs module

Provides some high level objects useful to use the Atlas API.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from builtins import str

from atlasapi.lib import logger

from atlasapi.settings import Settings
from atlasapi.errors import ErrRole

from enum import Enum
from dateutil import parser
from atlasapi.atlas_types import *
from typing import Optional, NewType, List, Union, Iterable, BinaryIO, Any
from datetime import datetime
from atlasapi.lib import AtlasGranularities, AtlasPeriods, AtlasLogNames
import logging
from future import standard_library
from logging import Logger

from atlasapi.measurements import AtlasMeasurementTypes, AtlasMeasurementValue, AtlasMeasurement

standard_library.install_aliases()
logger: Logger = logging.getLogger('Atlas.specs')


# etc., as needed


class IAMType(Enum):
    NONE = 'None'  # The user does not use AWS IAM credentials.
    USER = 'USER'  # New database user has AWS IAM user credentials.
    ROLE = 'ROLE'  # New database user has credentials associated with an AWS IAM role.


class ReplicaSetTypes(Enum):
    REPLICA_PRIMARY = 'ReplicaSet primary'
    REPLICA_SECONDARY = 'ReplicaSet secondary'
    RECOVERING = 'Recovering'
    SHARD_MONGOS = 'Mongos router'
    SHARD_CONFIG = 'Config server'
    SHARD_STANDALONE = 'Standalone'
    SHARD_PRIMARY = 'Shard primary'
    SHARD_SECONDARY = 'Shard secondary'
    NO_DATA = 'No data available'
    SHARD_CONFIG_PRIMARY = 'Config server'
    SHARD_CONFIG_SECONDARY = 'Config server'


class HostLogFile(object):
    def __init__(self, log_name: AtlasLogNames = None, log_file_binary: BinaryIO = None):
        """Container for Atlas log files for a host instance

        Args:
            log_name (AtlasLogName): They type of log file
            log_file_binary (BinaryIO): The binary of the gzipped log file
        """
        self.log_file_binary = log_file_binary
        self.log_name = log_name


class Host(object):
    def __init__(self, data: dict) -> None:
        """An Atlas Host
            Contains all basic information for an Atlas host.
            Using various methods, enables the retrieval of more advanced data, such as measurements and log files.

        Args:
            data(dict): An Atlas format host data dictionary.

        Attributes:
            created (datetime): The datetime the host was created.
            group_id (str): The Atlas group(project) id that this host belongs to
            hostname (str): The fqdn hostname
            hostname_alias (str) : User-friendly hostname of the cluster node. The user-friendly hostname is typically
             the standard hostname for a cluster node and it appears in the connection string for a cluster instead of
             the value of the hostname field.
            id (str): The internal atlas id of the host.
            last_ping (Union[datetime,str]): The datetime of the last Automation Agent ping
            links (List[str]): A list of internal reference links for the host
            port (int): The TCP port the instance is running on.
            replica_set_name (str): The name of the replica set running on the instance
            type (ReplicaSetTypes): The type of replica set this intstance is a member of
            measurements (Optional[List[measurements.AtlasMeasurement]]: Holds list of host Measurements
            cluster_name (str): The cluster name (taken from the hostname)
            cluster_name_alias (str) : The cluster name user alias, taken from the hostname user friendly alias.
            log_files (Optional[List[HostLogFile]]): Holds list of log files when requested.
        """
        if type(data) != dict:
            raise NotADirectoryError('The data parameter must be ann dict, you sent a {}'.format(type(data)))
        else:
            try:
                self.created: datetime = parser.parse(data.get("created", None))
            except (ValueError, OverflowError):
                self.created = data.get("created", None)
            self.group_id: str = data.get("groupId", None)
            self.hostname: str = data.get("hostname", None)
            self.hostname_alias: str = data.get("userAlias", self.hostname)
            self.id: str = data.get("id", None)
            try:
                self.last_ping = parser.parse(data.get("lastPing", None))
            except (ValueError, OverflowError, TypeError):
                self.last_ping = data.get("lastPing", None)
            self.links: Optional[List[str]] = data.get("links", [])
            self.port: Optional[int] = data.get("port", None)
            self.replica_set_name: Optional[str] = data.get("replicaSetName", None)
            self.type: OptionalStr = ReplicaSetTypes[data.get("typeName", "NO_DATA")]
            self.measurements: Optional[List[AtlasMeasurement]] = []

            # NOTE: cluster_name_alias is NOT reliable since it relies on parsing the hostname_alias, which is
            # lowercased. If the clustername_alias uses any casing, this will not be usuable.

            if '-shard-' in self.hostname_alias:
                self.cluster_name_alias: str = self.hostname_alias.split('-shard-')[0]
            elif '-config-' in self.hostname_alias:
                self.cluster_name_alias: str = self.hostname_alias.split('-config-')[0]
            else:
                self.cluster_name_alias: str = self.hostname_alias.split('-')[0]

            if '-shard-' in self.hostname:
                self.cluster_name: str = self.hostname.split('-shard-')[0]
            elif '-config-' in self.hostname:
                self.cluster_name: str = self.hostname.split('-config-')[0]
            else:
                self.cluster_name: str = self.hostname.split('-')[0]

            self.log_files: Optional[List[HostLogFile]] = None

    def get_measurement_for_host(self, atlas_obj, granularity: Optional[AtlasGranularities] = None,
                                 period: Optional[AtlasPeriods] = None,
                                 measurement: Optional[AtlasMeasurementTypes] = None,
                                 iterable: bool = True) -> Union[dict, Iterable[AtlasMeasurement]]:
        """Get  measurement(s) for a host

        Returns measurements for the Host object.

        url: https://docs.atlas.mongodb.com/reference/api/process-measurements/


        Accepts either a single measurement, but will retrieve more than one measurement
        if the measurement (using the AtlasMeasurementTypes class)

        /api/atlas/v1.0/groups/{GROUP-ID}/processes/{HOST}:{PORT}/measurements

        Keyword Args:
            host_obj (Host): the host
            granularity (AtlasGranularities): the desired granularity
            period (AtlasPeriods): The desired period
            measurement (AtlasMeasurementTypes) : The desired measurement or Measurement class
            iterable (bool): To return an iterable high level object instead of a low level API response

        Returns:
             Iterable[AtlasMeasurement] or dict: Iterable object representing this function OR Response payload

        Raises:


        """

        # Set default
        if measurement is None:
            measurement = AtlasMeasurementTypes.Cache.dirty
            logger.info(f'The measurement is {measurement}')
        if period is None:
            period = AtlasPeriods.WEEKS_1
        logger.info(f'The granularity is {granularity}')

        if granularity is None:
            granularity = AtlasGranularities.HOUR
        logger.info(f'The granularity is {granularity}')

        # Check to see if we received a leaf or branch of the measurements
        try:
            parent = super(measurement)
            logger.info('We received a branch, whos parent is {}'.format(parent.__str__()))
            leaves = measurement.get_all()
            measurement_list = list(leaves)
            measurement = '&m='.join(measurement_list)
        except TypeError:
            logger.info('We received a leaf')

        # Build the URL
        uri = Settings.api_resources["Monitoring and Logs"]["Get measurement for host"].format(
            group_id=self.group_id,
            host=self.hostname,
            port=self.port,
            granularity=granularity,
            period=period,
            measurement=measurement
        )
        logger.info(f'The URI is.... {uri}')

        # Build the request
        return_val = atlas_obj.network.get(Settings.BASE_URL + uri)
        measurement_obj = None
        if iterable:
            measurements = return_val.get('measurements')
            measurements_count = len(measurements)
            logger.info('There are {} measurements.'.format(measurements_count))

            for each in measurements:
                measurement_obj = AtlasMeasurement(name=each.get('name'),
                                                   units=each.get('units', None),
                                                   period=period,
                                                   granularity=granularity)
                for each_and_every in each.get('dataPoints'):
                    measurement_obj.measurements = AtlasMeasurementValue(each_and_every)

            yield measurement_obj

        else:
            return return_val

    def add_measurements(self, measurement) -> None:
        # TODO: Make measurements unique, use a set instead, but then how do we concat 2?
        self.measurements = self.measurements + measurement

    def add_log_file(self, name: AtlasLogNames, file: BinaryIO) -> None:
        """
        Adds the passed log file to the hosts object

        Args:
            name (AtlasLogNames): The type of logfile to be appended.
            file (BinaryIO): The file to be appended
        """
        log_obj = HostLogFile(log_name=name, log_file_binary=file)
        if self.log_files is None:
            self.log_files = [log_obj]
        else:
            self.log_files.append(log_obj)

    def get_partitions(self, atlas_obj) -> Iterable[str]:
        """Returns names of all disks(partitions) configured on the Atlas Host
        Args:
            atlas_obj :

        Returns:
            Iterable[str]: A list of partition names.
        """
        uri = Settings.api_resources["Monitoring and Logs"]["Get Available Disks for Process"].format(
            group_id=self.group_id,
            host=self.hostname,
            port=self.port,
        )
        logger.info(f"The full URI being called is {Settings.BASE_URL + uri}")
        return_val = atlas_obj.network.get(Settings.BASE_URL + uri)
        for each_partition in return_val.get("results"):
            partition_name: str = each_partition.get('partitionName', None)
            yield partition_name

    def get_measurements_for_disk(self, atlas_obj, partition_name: str,
                                  granularity: Optional[AtlasGranularities] = None,
                                  period: Optional[AtlasPeriods] = None, iterable: bool = True) -> \
            Iterable[Union[AtlasMeasurement, Any]]:
        """Returns All Metrics for a Hosts partition, for a given period and granularity.

        Uses default granularity and period if not passed.

        Args:
            iterable (bool): Defaults to true, if not true will return the raw response from API.
            partition_name: The Atlas partition name (commonly `data`)
            period (Optional[AtlasPeriods]):The period for the disk measurements
            granularity (Optional[AtlasGranularitues]): The granularity for the disk measurements.
            atlas_obj (atlasapi.atlas.Atlas): A configured Atlas instance to connect to the API with.

        Returns:
            List[str]: A list of partition names.
        """
        if period is None:
            period = AtlasPeriods.WEEKS_1
        logger.info(f'The period  is {period}')

        if granularity is None:
            granularity = AtlasGranularities.HOUR
        logger.info(f'The granularity is {granularity}')

        parameters = {'granularity': granularity, 'period': period}
        uri = Settings.api_resources["Monitoring and Logs"]["Get Measurements of a Disk for Process"].format(
            group_id=self.group_id,
            host=self.hostname,
            port=self.port,
            disk_name=partition_name,
        )
        logger.info(f"The full URI being called is {Settings.BASE_URL + uri}")
        logger.info(f"We sent the following parameters: {parameters}")
        return_val = atlas_obj.network.get_big(Settings.BASE_URL + uri, params=parameters)

        measurement_obj = None
        if iterable:
            measurements = return_val.get('measurements')
            measurements_count = len(measurements)
            logger.warning('There are {} measurements.'.format(measurements_count))
            for each in measurements:
                measurement_obj = AtlasMeasurement(name=each.get('name'),
                                                   period=period,
                                                   granularity=granularity,
                                                   units=each.get('units', None))
                for each_and_every in each.get('dataPoints'):
                    measurement_obj.measurements = AtlasMeasurementValue(each_and_every)

                yield measurement_obj

        return return_val

    def data_partition_stats(self, atlas_obj, granularity: Optional[AtlasGranularities] = None,
                             period: Optional[AtlasPeriods] = None, ) -> Iterable[AtlasMeasurement]:
        """Returns disk measurements for the data partition of the host.

        Hard codes the name of the partition to `data` and returns all metrics.

        Args:
            atlas_obj: Instantiated Atlas instance to access the API
            granularity (Optional[AtlasGranularitues]): The granularity for the disk measurements.
            atlas_obj (atlasapi.atlas.Atlas): A configured Atlas instance to connect to the API with.

        Returns (Iterable[AtlasMeasurement]): A generator yielding AtlasMeasurements

        """
        return self.get_measurements_for_disk(atlas_obj=atlas_obj, partition_name='data', granularity=granularity,
                                              period=period)

    def get_databases(self, atlas_obj) -> Iterable[str]:
        """Returns all disks(partitions) configured on the Atlas Host

        Yields names of databases, and appends them to the databa

        Args:
            atlas_obj :

        Returns:
            List[str]: A list of database names.
        """
        uri = Settings.api_resources["Monitoring and Logs"]["Get Available Databases for Process"].format(
            group_id=self.group_id,
            host=self.hostname,
            port=self.port,
        )
        logger.info(f"The full URI being called is {Settings.BASE_URL + uri}")
        return_val = atlas_obj.network.get(Settings.BASE_URL + uri)
        for each_database in return_val.get("results"):
            db_name = each_database.get('databaseName', None)
            yield db_name

    def get_measurements_for_database(self, atlas_obj, database_name: str,
                                      granularity: Optional[AtlasGranularities] = None,
                                      period: Optional[AtlasPeriods] = None, iterable: bool = True) -> \
            Iterable[Union[AtlasMeasurement, Any]]:
        """Returns All Metrics for a database, for a given period and granularity.

        Uses default granularity and period if not passed.

        Args:
            iterable (bool): Defaults to true, if not true will return the raw response from API.
            database_name (str): The database name (local should always exist, and can be used for testing)
            period (Optional[AtlasPeriods]):The period for the disk measurements
            granularity (Optional[AtlasGranularities]): The granularity for the disk measurements.
            atlas_obj (atlasapi.atlas.Atlas): A configured Atlas instance to connect to the API with.

        Returns:
           Iterable[Union[AtlasMeasurement, Any]: Yields AtlasMeasirements or the original response.
        """
        if period is None:
            period = AtlasPeriods.WEEKS_1
        logger.info(f'The period is {period}')

        if granularity is None:
            granularity = AtlasGranularities.HOUR
        logger.info(f'The granularity is {granularity}')

        parameters = {'granularity': granularity, 'period': period}
        uri = Settings.api_resources["Monitoring and Logs"]["Get Measurements of a Database for Process"].format(
            group_id=self.group_id,
            host=self.hostname,
            port=self.port,
            database_name=database_name,
        )
        logger.info(f"The full URI being called is {Settings.BASE_URL + uri}")
        logger.info(f"We sent the following parameters: {parameters}")
        return_val = atlas_obj.network.get_big(Settings.BASE_URL + uri, params=parameters)

        measurement_obj = None
        if iterable:
            measurements = return_val.get('measurements')
            measurements_count = len(measurements)
            logger.warning('There are {} measurements.'.format(measurements_count))
            for each in measurements:
                measurement_obj = AtlasMeasurement(name=each.get('name'),
                                                   period=period,
                                                   granularity=granularity,
                                                   units=each.get('units', None))
                for each_and_every in each.get('dataPoints'):
                    measurement_obj.measurements = AtlasMeasurementValue(each_and_every)

                yield measurement_obj

        return return_val

    def __hash__(self):
        return hash(self.hostname)

    def __eq__(self, other):
        if isinstance(other, Host):
            return self.hostname == other.hostname


ListOfHosts = NewType('ListOfHosts', List[Optional[Host]])


class RoleSpecs:
    """Roles supported by Atlas"""
    atlasAdmin = "atlasAdmin"
    readWriteAnyDatabase = "readWriteAnyDatabase"
    readAnyDatabase = "readAnyDatabase"
    backup = "backup"
    clusterMonitor = "clusterMonitor"
    dbAdmin = "dbAdmin"
    dbAdminAnyDatabase = "dbAdminAnyDatabase"
    enableSharding = "enableSharding"
    read = "read"
    readWrite = "readWrite"


class DatabaseUsersPermissionsSpecs:
    """Permissions spec for Database User

    Constructor

    Args:
        username (str): Username of the DB
        password (str): Password for the username
        aws_iam_type (IAMType): AWS IAM method by which the database applies IAM credentials to authenticates the database
         user. Atlas defaults to NONE. (optional)

    Keyword Args:
        databaseName (str): Auth Database Name
    """

    def __init__(self, username: str, password: str = None,
                 aws_iam_type: Optional[IAMType] = None, databaseName=Settings.databaseName) -> None:
        self.username = username
        self.password = password
        self.aws_iam_type = aws_iam_type.value
        self.databaseName = databaseName
        self.roles = []

    def getSpecs(self) -> dict:
        """Get specs

        Returns:
            dict: Representation of the object
        """
        content = dict(databaseName=self.databaseName, roles=self.roles, username=self.username)
        if self.password:
            content["password"] = self.password
            content["awsIAMType"] = "NONE"
        elif self.aws_iam_type:
            content["awsIAMType"] = self.aws_iam_type
            content["databaseName"] = r"$external"

        return content

    def add_roles(self, databaseName: str, roleNames: List[RoleSpecs], collectionName: str = None):
        """Add multiple roles

        Args:
            databaseName (str): Database Name
            roleNames (list of RoleSpecs): roles

        Keyword Args:
            collectionName (str): Collection

        Raises:
            ErrRoleException: role not compatible with the databaseName and/or collectionName
            :param databaseName: Database Name
            :param roleNames: roles
            :param collectionName: Collection

        """
        for roleName in roleNames:
            self.add_role(databaseName, roleName, collectionName)

    def add_role(self, databaseName: str, roleName: str, collectionName: OptionalStr = None):
        """Add one role

        Args:
            databaseName (str): Database Name
            roleName (RoleSpecs): role

        Keyword Args:
            collectionName (str): Collection

        Raises:
            ErrRole: role not compatible with the databaseName and/or collectionName
            :param roleName:
            :param databaseName:
            :type collectionName: str

        TODO: Need to test if this works correctly, looks like their may be a type problem.
        """
        role = {"databaseName": databaseName,
                "roleName": roleName}

        if collectionName:
            role["collectionName"] = collectionName

        # Check atlas constraints
        if collectionName and roleName not in [RoleSpecs.read, RoleSpecs.readWrite]:
            raise ErrRole("Permissions [%s] not available for a collection" % roleName)
        elif not collectionName and roleName not in [RoleSpecs.read, RoleSpecs.readWrite,
                                                     RoleSpecs.dbAdmin] and databaseName != "admin":
            raise ErrRole("Permissions [%s] is only available for admin database" % roleName)

        if role not in self.roles:
            self.roles.append(role)

    def remove_roles(self, databaseName, roleNames, collectionName=None):
        """Remove multiple roles

        Args:
            collectionName (str):
            databaseName (str): Database Name
            roleNames (list of RoleSpecs): roles

        Keyword Args:
            collectionName (str): Collection
        """
        for roleName in roleNames:
            self.remove_role(databaseName, roleName, collectionName)

    def remove_role(self, databaseName, roleName, collectionName=None):
        """Remove one role

        Args:
            databaseName (str): Database Name
            roleName (RoleSpecs): role

        Keyword Args:
            collectionName (str): Collection
        """
        role = {"databaseName": databaseName,
                "roleName": roleName}

        if collectionName:
            role["collectionName"] = collectionName

        if role in self.roles:
            self.roles.remove(role)

    def clear_roles(self):
        self.roles.clear()


class DatabaseUsersUpdatePermissionsSpecs(DatabaseUsersPermissionsSpecs):
    """Update Permissions spec for Database User

    Constructor

    Keyword Args:
        password (str): Password for the username
    """

    def __init__(self, password=None):
        super().__init__(None, password)

    def getSpecs(self):
        """Get specs

        Returns:
            dict: Representation of the object
        """

        content = {}

        if len(self.roles) != 0:
            content["roles"] = self.roles

        if self.password:
            content["password"] = self.password

        return content


class AlertStatusSpec:
    """Alert Status"""
    TRACKING = "TRACKING"
    OPEN = "OPEN"
    CLOSED = "CLOSED"



#!/usr/bin/env python3
from setuptools import setup, find_packages

setup(
    name='atlasapi',
    version='2.0.1',
    python_requires='>=3.7',
    packages=find_packages(exclude=("tests",)),
    install_requires=['requests', 'python-dateutil', 'isodate', 'future', 'pytz','coolname', 'humanfriendly', 'nose'],
    setup_requires=['wheel'],
    # Metadata
    author="Matthew G. Monteleone",
    author_email="mgm@mgm.dev",
    license="Apache License 2.0",
    description="Expose MongoDB Atlas Cloud provider APIs",
    long_description=open('README.rst').read(),
    url="https://github.com/mgmonteleone/python-atlasapi",
    keywords=["atlas", "mongo", "mongodb", "cloud", "api"],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: Apache Software License',

        'Operating System :: OS Independent',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    entry_points={
        'console_scripts': [
            #final script name:local package name:local function name
            'atlascli=atlascli.cli:main',
        ]
    },
    extras_require={}

)



# Copyright (c) 2019 Joe Drumgoole
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from requests.exceptions import HTTPError


class AtlasError(HTTPError):
    def __init__(self, *args, **kwargs):
        self._text = kwargs.pop("text", None)

        super().__init__(*args, **kwargs)

    @property
    def text(self):
        return self._text


class AtlasAuthenticationError(AtlasError):
    pass


class AtlasGetError(AtlasError):
    pass


class AtlasPostError(AtlasError):
    pass


class AtlasPatchError(AtlasError):
    pass


class AtlasDeleteError(AtlasError):
    pass


class AtlasEnvironmentError(ValueError):
    pass


class AtlasInitialisationError(ValueError):
    pass



from typing import Optional


class Organization:
    def __init__(self, name: str, is_deleted: bool = False, links: Optional[list] = None, id: Optional[str] = None):
        self.id = id
        self.links = links
        self.is_deleted = is_deleted
        self.name = name

    @classmethod
    def from_dict(cls, data_dict: dict):
        return cls(data_dict.get("name"), data_dict.get("isDeleted", False), data_dict.get("links", []),
                   data_dict.get("id", None))



# Copyright (c) 2020 Matthew G. Monteleone
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from typing import Tuple, NewType, List, Optional

from dateutil.parser import parse
from enum import Enum
from pprint import pprint
from atlasapi.lib import logger
from atlasapi.whitelist import WhitelistEntry
import ipaddress
import requests
from awspublicranges.ranges import AwsIpRanges, AwsPrefix


class Roles(Enum):
    """Enum of the roles which can be assigned to a Atlas Public API Key

    """
    ORG_OWNER = 'ORG_OWNER'.replace('_', ' ').capitalize()
    ORG_MEMBER = 'ORG_OWNER'.replace('_', ' ').capitalize()
    ORG_GROUP_CREATOR = 'ORG_GROUP_CREATOR'.replace('_', ' ').capitalize()
    ORG_BILLING_ADMIN = 'ORG_BILLING_ADMIN'.replace('_', ' ').capitalize()
    ORG_READ_ONLY = 'ORG_READ_ONLY'.replace('_', ' ').capitalize()

    GROUP_CHARTS_ADMIN = 'GROUP_CHARTS_ADMIN'.replace('_', ' ').capitalize()
    GROUP_CLUSTER_MANAGER = 'GROUP_CLUSTER_MANAGER'.replace('_', ' ').capitalize()
    GROUP_DATA_ACCESS_ADMIN = 'GROUP_DATA_ACCESS_ADMIN'.replace('_', ' ').capitalize()
    GROUP_DATA_ACCESS_READ_ONLY = 'GROUP_DATA_ACCESS_READ_ONLY'.replace('_', ' ').capitalize()
    GROUP_DATA_ACCESS_READ_WRITE = 'GROUP_DATA_ACCESS_READ_WRITE'.replace('_', ' ').capitalize()
    GROUP_OWNER = 'GROUP_OWNER'.replace('_', ' ').capitalize()
    GROUP_READ_ONLY = 'GROUP_OWNER'.replace('_', ' ').capitalize()


class ApiKeyRoles(object):
    def __init__(self, group_id: str, org_id: str, role_name: Roles):
        """API roles assigned to an API key.

        Args:
            group_id:
            org_id:
            role_name: The role name itself.
        """
        self.role_name: Roles = role_name
        self.org_id: str = org_id
        self.group_id: str = group_id

    @classmethod
    def fill_from_dict(cls, data_dict: dict):
        """
        Fills the object from an Atlas Dict

        :param data_dict: A dict as returned from Atlas
        :return:
        """
        group_id = data_dict.get('groupId', None)
        org_id = data_dict.get('orgId', None)
        role_name: Roles = data_dict.get('roleName', None)

        return cls(group_id, org_id, role_name)


class ApiKey(object):
    def __init__(self, desc: str = None, id: str = None, private_key: str = None, public_key: str = None,
                 roles: List[ApiKeyRoles] = None):
        """An Atlas Pubic API access key

        Includes the roles assigned to each key.

        Args:
            desc:
            id:
            private_key:
            public_key:
            roles:
        """
        self.roles = roles
        self.public_key = public_key
        self.private_key = private_key
        self.id = id
        self.desc = desc

    @classmethod
    def fill_from_dict(cls, data_dict: dict):
        """
        Fills the object from an Atlas Dict

        :param data_dict: A dict as returned from Atlas
        :return:
        """
        desc = data_dict.get('desc', None)
        public_key = data_dict.get('publicKey', None)
        private_key = data_dict.get('privateKey', None)
        id = data_dict.get('id', None)
        roles_raw: List[dict] = data_dict.get('roles', None)
        roles: Optional[List[ApiKeyRoles]] = []
        for each_role in roles_raw:
            roles.append(ApiKeyRoles.fill_from_dict(data_dict=each_role))

        return cls(desc, id, private_key, public_key, roles)



from typing import Optional, List


class AtlasUser:
    def __init__(self, country: str, email_address: str, first_name: str, last_name: str, roles: Optional[List[dict]],
                 username: Optional[str], links: Optional[List[str]] = None, mobile_number: str = None,
                 password: str = None, team_ids: Optional[List[str]] = None, id: str = None):
        self.username: str = username
        self.team_ids: Optional[List[str]] = team_ids
        self.roles: Optional[List[dict]] = roles
        self.password: str = password
        self.mobile_number: str = mobile_number
        self.links: Optional[List[str]] = links
        self.last_name: str = last_name
        self.id: Optional[str] = id
        self.first_name: str = first_name
        self.email_address: str = email_address
        self.country: str = country

    @classmethod
    def from_dict(cls, data_dict: dict):
        country = data_dict.get("country", None)
        email_address = data_dict.get("emailAddress", None)
        first_name = data_dict.get("firstName", None)
        last_name = data_dict.get("lastName", None)
        roles = data_dict.get("roles", [])
        username = data_dict.get("username", None)
        links = data_dict.get("links", None)
        mobile_number = data_dict.get("mobileNumber", None)
        password = data_dict.get("password", None)
        team_ids = data_dict.get("teamIds", None)
        id = data_dict.get("id", None)
        return cls(
            country, email_address, first_name, last_name, roles, username, links, mobile_number, password, team_ids,
            id)
